<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Requests;
use App\Models\User;
use App\Models\RoomControlModel;
use App\Models\RoomModel;
use App\Models\GroupModel;
use App\Models\GroupTypeModel;
use App\Models\RightModel;
use App\Models\RightTypeModel;
use Auth;

class AdminController extends Controller_api {

    public function __construct() {
        parent::__construct();
        if ($this->authorize('PermissionChange', Auth::user())) {
            return redirect('home');
        };
    }

    /*
     * Slouží pro administrační rozhraní - oprávnění uživatelů, vrací šablonu
     */

    public function getPermView(Request $request) {
        $users = User::where('admin', false)->get();
        $rights = RightTypeModel::all();
        $class = $this->api->getAllClassrooms();
        $result = [];
        foreach ($class as $i => $cl) {
            $result[$i]['nazev'] = $cl;
        }
        //dd($all_groups);
        return view('permissions')->with(['classes' => $result, 'users' => $users, 'rights' => $rights]);
    }

    /*
     * Slouží pro administrační rozhraní - podbarvení uživatelů, vrací šablonu
     * v defaultu jsou 2 skupiny - admins a testy
     */

    public function getColouringView() {
        $admins_id = GroupTypeModel::where('name', 'admins')->first();
        $testy_id = GroupTypeModel::where('name', 'testy')->first();
        $admins = GroupModel::where('group_type_id', $admins_id->id)->get(); // vrací uživatele, kteří jsou typu admin
        $tests = GroupModel::where('group_type_id', $testy_id->id)->get();
        return view('colouring')->with(['admins' => $admins, 'tests' => $tests, 'admins_id' => $admins_id->id, 'testy_id' => $testy_id->id]);
    }

    /*
     * Vytvoří nového uživatele, který se bude podbarvovat(admin +testy) dle skupiny
     * vstup: id(group id) - int, user -string
     */

    public function addColouring(Request $request) {
        $new_user = new GroupModel;
        $new_user->group_type_id = $request->id;
        $new_user->user = $request->user;
        if ($new_user->save()) {
            $users = GroupModel::where('group_type_id', $request->id)->get();
            return response()->json(['users' => $users]);
        }
    }

    /*
     * Smaže uživatele, už se nebude podbarvovat
     * vstup: user - array of string
     */

    public function delColouring(Request $request) {
        foreach ($request->user as $user) {
            GroupModel::where('user', $user)->where('group_type_id', $request->id)->delete();
        }

        $users = GroupModel::where('group_type_id', $request->id)->get();
        return response()->json(['users' => $users]);
    }

    /*
     * Vrací oprávnění vstupu do místností a možnosti ovládat prvky aplikace(role)
     * vstup: id(user id) - int
     */

    public function getUserInfo(Request $request) {
        $rooms_access = RoomControlModel::where('user_id', $request->id)->with('getRoom')->get();
        $rights = RightModel::where('user_id', $request->id)->with('getType')->get();
        return response()->json(['rooms' => $rooms_access, 'rights' => $rights]);
    }

    /*
     * Změna oprávnění a přístupu
     * vstup: id(user id) - int, rights - array of int, rooms - array of string
     */

    public function modifyUser(Request $request) {
        $response = true;
        RightModel::where('user_id', $request->id)->delete();
        $rights = $request->rights;
        foreach ($rights as $right) {
            $new_right = new RightModel;
            $new_right->user_id = $request->id;
            $new_right->right_type_id = $right;
            if (!$new_right->save()) {
                $response = false;
            };
        }
        RoomControlModel::where('user_id', $request->id)->delete();
        $rooms = $request->rooms;
        foreach ($rooms as $room) {
            $room_desired = RoomModel::where('name', $room)->first();
            $new_room = new RoomControlModel;
            $new_room->user_id = $request->id;
            $new_room->room_id = $room_desired->id;
            if (!$new_room->save()) {
                $response = false;
            };
        }
        if ($response) {
            return response()->json(['msg' => 'Změna uložena'], 200);
        } else {
            return response()->json('error', 400);
        }
    }

}
