<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Requests;
use App\Http\Controllers\Controller;
use App\Models\GroupModel;
use App\Models\GroupTypeModel;
use App\Models\CommandModel;
use App\Models\RuleModel;
use SoapClient;

class ComputerController extends Controller_api {
    /*
     * Vrací styl podbarvení uživatele na základě loginu(zda je uveden ve skupinách)
     */

    private function getUserType($login) {
        $user = GroupModel::where('user', $login)->first();
        if (isset($user)) {
            $group = GroupTypeModel::find($user->group_type_id);
            if ($group->name == 'admins') {
                return 'user-admin';
            } else if ($group->name == 'testy') {
                return 'user-pef';
            }
        } else {
            return 'user-any';
        }
    }

    /*
     * Kontrola chybových stavů z API
     */

    private function checkComputerErrorState($autologin, $logged_user) {
        if ($autologin != 'f') {
            return true;
        }
        // chyba, když je login delší než 12 znaků
        if(strlen($logged_user) > 12){
            return true;
        }
        return false;
    }

    /*
     * Zajišťuje informace o počítačích (v pomyslné mřížce každé učebny v GUI)
     * vstup: název učebny
     */

    public function computerInfo(Request $request) {
        $result = array();
        $computers = $this->api->getPc($request->ucebna);
        $this->api->refreshInfo($computers);
        $infos = $this->api->getAllInfo($computers);
        for ($i = 0; $i < sizeof($infos); $i++) {
            $is_available = $infos[$computers[$i]]['dostupnost']; // 0=available
            $is_system = $infos[$computers[$i]]['system']; // číslo OS
            if ((!$is_available && $is_system == 3) || $is_available || $is_available == 2) {
                $result[$i]['dostupnost'] = 'pocitac-off'; // odpovída css stylu, který obarví šedě počítač
                $result[$i]['system'] = '';
                $result[$i]['prihlaseny'] = '';
            } elseif (!$is_available) {
                $result[$i]['dostupnost'] = 'pocitac-run';
                if ($is_system == 1) {
                    $result[$i]['system'] = 'fa fa-windows';
                } elseif ($is_system == 2) {
                    $result[$i]['system'] = 'fa fa-linux';
                } else {
                    $result[$i]['system'] = '';
                }
                if ($infos[$computers[$i]]['prihlas_uzivatel'] != '(null)' && strlen($infos[$computers[$i]]['prihlas_uzivatel']) < 12) { // je přihlášený uživatel
                    $result[$i]['prihlaseny'] = $infos[$computers[$i]]['prihlas_uzivatel'];
                    $result[$i]['group'] = $this->getUserType($infos[$computers[$i]]['prihlas_uzivatel']);
                } else {
                    $result[$i]['prihlaseny'] = '';
                }
            }
            $result[$i]['nazev'] = $infos[$computers[$i]]['nazev'];
            if ($this->checkComputerErrorState($infos[$computers[$i]]['autologin'],$infos[$computers[$i]]['prihlas_uzivatel'])) { // kontrola chybových stavů
                $result[$i]['dostupnost'] = 'pocitac-error';
            }
        }
        return response()->json(['data' => $result]);
    }

    /*
     * Akce po kliknutí na zapnout, nebo vypnout PC
     * vstup: command [power_on | power_off], computers - array, classes - array
     */

    public function changePowerState(Request $request) {
        if ($this->authorize('PcSwitch', $request->user())) {
            $computers = $request->computers;
            $classes = $request->classes; // učebny, ve kterých byly počítače vybrány (zde pouze z důvodu logování)
            if (!empty($computers) && $request->command === 'power_on') {
                $this->api->wakeUp($computers);
                $this->makeLog($computers, $classes, 'ZAP', $request->user()->login);
                return response()->json(array('msg' => 'Stroje se zapínají', 200, [], JSON_UNESCAPED_UNICODE));
            } else if (!empty($computers) && $request->command === 'power_off') {
                $this->api->shutdown($computers);
                $this->makeLog($computers, $classes, 'VYP', $request->user()->login);
                return response()->json(array('msg' => 'Stroje se vypínají'), 200, [], JSON_UNESCAPED_UNICODE);
            } else if (empty($computers)) {
                return response()->json(array('Nejsou vybrané stroje'), 400, [], JSON_UNESCAPED_UNICODE);
            } else {
                return response()->json(array('Někde je chyba'), 400, [], JSON_UNESCAPED_UNICODE);
            }
        } else {
            return response()->json(array('Nemáte oprávnění'), 403, [], JSON_UNESCAPED_UNICODE);
        }
    }

    /*
     * Přihlášení uživatele na cílové počítače
     * vstup: user - string, password - string, computers - array, class - array
     */

    public function userLog(Request $request) {

        if ($this->authorize('PcCommands', $request->user())) {
            if (!empty($request->computers)) {
                if (!empty($request->user && $request->pass)) {
                    $this->api->logMeIn($request->computers, $request->user, $request->pass);
                    $this->makeLog($request->computers, $request->classes, 'Log user', $request->user()->login);
                    return response()->json(array('msg' => 'Nastavuji automatické přihlašování'), 200, [], JSON_UNESCAPED_UNICODE);
                } else {
                    $this->api->logOut($request->computers);
                    $this->makeLog($request->computers, $request->classes, 'Logout user', $request->user()->login);
                    return response()->json(array('msg' => 'Ruším automatické přihlášení'), 200, [], JSON_UNESCAPED_UNICODE);
                }
            } elseif (empty($request->computers)) {
                return response()->json(array('Nejsou vybrané počítače'), 400, [], JSON_UNESCAPED_UNICODE);
            } else {
                return response()->json(array('Jiná chyba'), 400, [], JSON_UNESCAPED_UNICODE);
            }
        } else {
            return response()->json(array('Nemáte oprávnění'), 400, [], JSON_UNESCAPED_UNICODE);
        }
    }

    /*
     * Aktivuje režim přijímačky
     * vstupy: computers - array, classes - array
     */

    public function userLogPef(Request $request) {
        if ($this->authorize('TestSystem', $request->user())) {
            if (!empty($request->computers)) {
                $this->api->wakeUp($request->computers);
                $this->api->logPEF($request->computers);
                $this->makeLog($request->computers, $request->classes, 'Nastavuji přijímačkový režim', $request->user()->login);
                return response()->json(array('msg' => 'Přihlašuji'), 200, [], JSON_UNESCAPED_UNICODE);
            } elseif (empty($request->computers)) {
                return response()->json(array('Nejsou vybrané počítače'), 400, [], JSON_UNESCAPED_UNICODE);
            } else {
                return response()->json(array('Jiná chyba'), 400, [], JSON_UNESCAPED_UNICODE);
            }
        } else {
            return response()->json(array('Nemáte oprávnění'), 400, [], JSON_UNESCAPED_UNICODE);
        }
    }

    /*
     * Provedení příkazu na cílových počítačích
     * vstupy: computers - array, command - string, classes -array
     */

    public function doCommand(Request $request) {
        if ($this->authorize('PcCommands', $request->user())) {
            if (!empty($request->computers)) {
                $this->api->doCom($request->computers, $request->command);
                $this->makeLog($request->computers, $request->classes, 'Command', $request->user()->login);
                return response()->json(array('msg' => 'Příkaz proveden'), 200, [], JSON_UNESCAPED_UNICODE);
            } elseif (empty($request->computers)) {
                return response()->json(array('Nejsou vybrané počítače'), 400, [], JSON_UNESCAPED_UNICODE);
            } else {
                return response()->json(array('Jiná chyba'), 400, [], JSON_UNESCAPED_UNICODE);
            }
        } else {
            return response()->json(array('Nemáte oprávnění'), 400, [], JSON_UNESCAPED_UNICODE);
        }
    }

    /*
     * Vrací seznam dostupných příkazů, pokud proběhne seed DB, tak 
     * v defaultu vrací příkazy Zapnout pc a Vypnout pc
     */

    public function getCommandList(Request $request) {
        if ($this->authorize('PcCommands', $request->user())) {
            $commands_available = CommandModel::all();
            return response()->json(['commands' => $commands_available], 200);
        } else {
            return response()->json(array('Nemáte oprávnění'), 400, [], JSON_UNESCAPED_UNICODE);
        }
    }

    /*
     * Uložení a přidání nového příkazu
     * vstup: name - string, content - string, id - int
     */

    public function saveCommand(Request $request) {
        if ($this->authorize('PcCommands', $request->user())) {
            if (!empty($request->name) && !empty($request->content)) {
                if ($request->id == 'new') { // stav kdy id=new -> nový příkaz
                    $new_command = new CommandModel;
                    $new_command->name = $request->name;
                    $new_command->content = $request->content;
                    $new_command->save();
                    return response()->json(array('msg' => 'ok'), 200);
                } else {
                    $command = CommandModel::find($request->id);
                    $command->name = $request->name;
                    $command->content = $request->content;
                    $command->save();
                    return response()->json(array('msg' => 'ok'), 200);
                }
            } else {
                return response()->json(array('Prazdny vstup'), 400, [], JSON_UNESCAPED_UNICODE);
            }
        } else {
            return response()->json(array('Nemáte oprávnění'), 400, [], JSON_UNESCAPED_UNICODE);
        }
    }

    /*
     * Smaž příkaz
     * vstup: id - int
     */

    public function deleteCommand(Request $request) {
        if ($this->authorize('PcCommands', $request->user())) {
            if (!empty($request->id)) {
                $del = CommandModel::find($request->id);
                $del->delete();
                return response()->json(array('msg' => 'ok'), 200);
            } else {
                return response()->json(array('Prazdny vstup'), 400, [], JSON_UNESCAPED_UNICODE);
            }
        } else {
            return response()->json(array('Nemáte oprávnění'), 400, [], JSON_UNESCAPED_UNICODE);
        }
    }

    /*
     * Funkce smazání dat profilů na vzdálených PC
     * vstup: computers - array, classees - array
     */

    public function delProfile(Request $request) {

        if ($this->authorize('PcCommands', $request->user())) {
            if (!empty($request->computers)) {
                $this->api->delProf($request->computers);
                $this->makeLog($request->computers, $request->classes, 'Smazani uz. profilu na pc', $request->user()->login);
                return response()->json(array('msg' => 'Profily smazány'), 200, [], JSON_UNESCAPED_UNICODE);
            } elseif (empty($request->computers)) {
                return response()->json(array('Nejsou vybrané počítače'), 400, [], JSON_UNESCAPED_UNICODE);
            } else {
                return response()->json(array('Jiná chyba'), 400, [], JSON_UNESCAPED_UNICODE);
            }
        } else {
            return response()->json(array('Nemáte oprávnění'), 400, [], JSON_UNESCAPED_UNICODE);
        }
    }

    /*
     * Pískoviště
     */

    public function getComp() {
           
    }

    /*
     * Metoda pro vytvoření logu na serveru, pro každou učebnu se vloží jeden
     * záznam v logu(obsahuje uživatele, akci, seznam počítačů)
     */

    private function makeLog($computers, $classes, $action, $user) {
        $classes_included = array_unique($classes); // seznam učeben
        foreach ($classes_included as $i => $class_incl) {
            $pclog = ''; // seznam počítačů
            foreach ($classes as $j => $class) {
                if ($class_incl == $class) {
                    $pclog = $pclog . ' ' . $computers[$j];
                }
            }
            shell_exec('logger -t morpheus -i -p local0.info "' . $user . ' | ' . $action . ' | ' . $class_incl . ' | ' . $pclog . ' > /dev/null &"');
        }
    }

}
