<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Auth;
use App\Http\Requests;
use AuthenticatesAndRegistersUsers;
use App\Models\RoomModel;
use App\Models\ComputersModel;

class LoginController extends Controller_api {
    /*
     * Vlastní autentizační driver s připojením na LDAP byl vytvořen dle návodu na
     * https://blog.georgebuckingham.com/laravel-52-auth-custom-user-providers-drivers/
     */

    protected $redirectTo = '/';

    public function __construct() {
        parent::__construct();
        $this->middleware('guest', ['except' => 'logout']);
    }

    public function loginForm() {
        return view('auth.login');
    }

    public function login(Request $request) {
        if (empty($request->user()->login)) {
            if (Auth::attempt(['login' => $request->login, 'password' => $request->password])) {
                $this->updateRoomDb();
                $this->updateComputerDb();
                return redirect()->intended('/');
            } else {
                return redirect('/login')->withErrors(['login' => $request->login, 'msg' => 'Špatné jméno, nebo heslo']);
            }
        } else {
            return redirect('/');
        }
    }

    public function logout() {
        Auth::logout();
        return redirect()->route('login');
    }

    // CACHING DATABAZE
    /*
     * Databáze počítačů a učeben se plní automaticky při prvním přihlášení uživatele do aplikace (admin_local)
     * Při přihlášení každého uživatele se zkontroluje aktuální databáze s odpovědí API
     * Hledají se diference a podle nich se upravuje DB aplikace(v případě přidání počítačů, nebo učeben na API)
     */
    public function updateRoomDb() {
        $db_array = array();
        $api_rooms = $this->api->getAllClassrooms();
        $db_rooms = RoomModel::all('name');
        foreach ($db_rooms as $db_room) {
            $db_array[] = $db_room->name;
        }
        if (!empty($api_rooms)) {
            $this->updateDbByLeftSet($api_rooms, $db_array, 'create_new', 'room');
            $this->updateDbByLeftSet($db_array, $api_rooms, 'delete_old', 'room');
        }
    }

    public function updateComputerDb() {
        $api_rooms = array();
        $classes = $this->api->getAllClassrooms();
        foreach ($classes as $class) {
            $temp = $this->api->getPc($class);
            if (!empty($temp)) {
                $api_rooms = array_merge($api_rooms, $temp);
            }
        }
        $db_array = array();
        $db_rooms = ComputersModel::all('name');
        foreach ($db_rooms as $db_room) {
            $db_array[] = $db_room->name;
        }
        if (!empty($api_rooms)) {
            $this->updateDbByLeftSet($api_rooms, $db_array, 'create_new', 'pc');
            $this->updateDbByLeftSet($db_array, $api_rooms, 'delete_old', 'pc');
        }
    }

    /*
     * Porovnává množiny dle první(left), zadaná operace: 1. přidává do DB přebytek nebo 2. odebírá rozdíl
     * vstup: left_set - array of string, right_set - array of string, operation - [create_new | delete_old], model - [pc | room]
     */

    private function updateDbByLeftSet($left_set, $right_set, $operation, $model) {
        foreach ($left_set as $left_item) {
            $item_exists = false;
            foreach ($right_set as $right_item) {
                if ($left_item == $right_item) {
                    $item_exists = true;
                    break;
                }
            }
            if (!$item_exists && $operation == 'create_new') {
                if ($model == 'room') {
                    $item = new RoomModel;
                } else if ($model == 'pc') {
                    $item = new ComputersModel;
                }
                $item->name = $left_item;
                $item->save();
            } else if (!$item_exists && $operation == 'delete_old') {
                if ($model == 'room') {
                    RoomModel::where('name', $left_item)->delete();
                } else if ($model == 'pc') {
                    ComputersModel::where('name', $left_item)->delete();
                }
            }
        }
    }

}
