<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Requests;
use App\Http\Controllers\Controller;
use App\Models\RuleModel;
use App\Models\RuleBindingModel;
use App\Models\ComputersModel;
use Auth;
use App\Models\RoomModel;
use App\Models\CommandModel;

class RuleController extends Controller_api {

    public function __construct() {
        parent::__construct();
        if (!$this->authorize('RulesEdit', Auth::user())) {
            return response()->json('not allowed', 300);
        }
    }

    /*
     * Převádí vlastnosti pravidel(z DB) do formátu pro GUI(názvy stylů)
     */

    private function prepareRules($rules) {

        foreach ($rules as $rule) {
            $computers = array();
            if ($rule->class) {
                $rule->class = '&bull;';
            } else {
                $rule->class = '';
            }
            $command = CommandModel::find($rule->command_id);
            $rule->command = $command->name;

            if ($rule->active) {
                $rule->tlacitko = 'pause';
                $rule->pause = '';
            } else {
                $rule->tlacitko = 'play';
                $rule->pause = 'pravidlo_pause';
            }
            if ($rule->Mon) {
                $rule->po = 'date_active';
            } else {
                $rule->po = '';
            }
            if ($rule->Tue) {
                $rule->ut = 'date_active';
            } else {
                $rule->ut = '';
            }
            if ($rule->Wed) {
                $rule->st = 'date_active';
            } else {
                $rule->st = '';
            }
            if ($rule->Thu) {
                $rule->ct = 'date_active';
            } else {
                $rule->ct = '';
            }
            if ($rule->Fri) {
                $rule->pa = 'date_active';
            } else {
                $rule->pa = '';
            }
            if ($rule->Sat) {
                $rule->so = 'date_active';
            } else {
                $rule->so = '';
            }
            if ($rule->Sun) {
                $rule->ne = 'date_active';
            } else {
                $rule->ne = '';
            }
        }

        return $rules;
    }

    /*
     * Vrací seznam pravidel v DB
     */

    public function getRules() {
        $rules = RuleModel::with('computersIncluded.computerName')->orderBy('created_at')->get();

        $result = $this->prepareRules($rules);
        return response()->json(array('rules' => $result), 200);
    }

    /*
     * Přidává počítače do DB tabulky RuleBinding a asociuje je s pravidlem
     * vstup: computers - array, rule_id - int
     */

    private function insertIncludedComputers($computers, $rule_id) {
        // zkontrolujeme, jestli probiha modifikace, kdyz ano tak smazu stare pocitace
        $old_computers = RuleBindingModel::where('rules_id', $rule_id);
        if (!empty($old_computers)) {
            $old_computers->delete();
        }
        $success = true;
        if (!empty($computers)) {
            foreach ($computers as $computer) {
                $pc_id = ComputersModel::where('name', $computer)->get();
                $record = new RuleBindingModel;
                $record->rules_id = $rule_id;
                $record->computers_id = $pc_id[0]->id;
                if (!$record->save()) {
                    $success = false;
                }
            }
        }
        return $success;
    }

    /*
     * Přidává nové pravidlo
     * vstup: days - array of bool, nazev - string, kdy - string, command_id - int, computers - array, classes - array
     */

    public function addRule(Request $request) {
        $days = array();
        $days = $request->days;
        $rule = new RuleModel();
        $rule->name = $request->nazev;
        $rule->time = $request->kdy;
        $rule->class = FALSE;
        $rule->command_id = $request->command_id;
        $rule->active = TRUE;
        $rule->Mon = $days[0];
        $rule->Tue = $days[1];
        $rule->Wed = $days[2];
        $rule->Thu = $days[3];
        $rule->Fri = $days[4];
        $rule->Sat = $days[5];
        $rule->Sun = $days[6];
        if ($rule->save() && $this->insertIncludedComputers($request->computers, $rule->id)) {
            $this->makeLog($request->computers, $request->classes, 'Make rule', $request->user()->login, $rule->name);
            $id = $rule->id;
            $result = RuleModel::with('computersIncluded.computerName')->where('id', $id)->get();
            $result = $this->prepareRules($result);
            return response()->json(['rules' => $result], 200); // vraci vložené pravidlo a jeho počítače
        } else {
            return response()->json(array('Pravidlo nelze pridat'), 400);
        }
    }

    /*
     * Maže pravidlo
     */

    public function deleteRule(Request $request) {
        $rule = RuleModel::find($request->id);
        if (!empty($rule) && $rule->delete()) {
            return response()->json(array('msg' => 'Pravidlo smazano'), 200);
        } else {
            return response()->json(array('Smazani se nazdarilo'), 400);
        }
    }

    /*
     * Modifikace pravidla
     * vstup: days - array of bool, nazev - string, kdy - string, command_id - int, computers - array, classes - array, active["" | *]
     */

    public function modifyRule(Request $request) {

        $rule = RuleModel::find($request->id);
        if (!empty($rule)) {
            $days = array();
            $days = $request->days;
            $active = ($request->active == '' ? TRUE : FALSE);
            $rule->name = $request->nazev;
            $rule->time = $request->kdy;
            $rule->active = $active;
            $rule->command_id = $request->command_id;
            $rule->Mon = $days[0];
            $rule->Tue = $days[1];
            $rule->Wed = $days[2];
            $rule->Thu = $days[3];
            $rule->Fri = $days[4];
            $rule->Sat = $days[5];
            $rule->Sun = $days[6];

            if ($rule->save() && $this->insertIncludedComputers($request->computers, $rule->id)) {
                $result = RuleModel::with('computersIncluded.computerName')->where('id', $rule->id)->get();
                $result = $this->prepareRules($result);
                return response()->json(array('rules' => $result), 200);
            } else {
                return response()->json(array('Nekde je chyba'), 400);
            }
        } else {
            return response()->json(array('Nekde je chyba'), 400);
        }
    }

    /*
     * Metoda pro vytvoření logu na serveru, pro každou učebnu se vloží jeden
     * záznam v logu(obsahuje uživatele, akci, seznam počítačů)
     */

    private function makeLog($computers, $classes, $action, $user, $description) {
        $classes_included = array_unique($classes);
        foreach ($classes_included as $i => $class_incl) {
            $pclog = '';
            foreach ($classes as $j => $class) {
                if ($class_incl == $class) {
                    $pclog = $pclog . ' ' . $computers[$j];
                }
            }
            exec('logger -t morpheus -i -p local0.info "' . $user . ' | ' . $action . ' | ' . $class_incl . ' | ' . $pclog . ' | ' . $description . '"');
        }
    }

}
